#ifndef CYGONCE_HAL_PLATFORM_SETUP_H
#define CYGONCE_HAL_PLATFORM_SETUP_H

/*=============================================================================
//
//      hal_platform_setup.h
//
//      Platform specific support for HAL (assembly code)
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: gthomas
//               Xavier Debreuil <xavier.debreuil@inventel.fr>
// Date:         2000-05-08
// Purpose:      Intel SA1110/Blue_2g platform specific support routines
// Description: 
// Usage:        #include <cyg/hal/hal_platform_setup.h>
//     Only used by "vectors.S"         
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

#include <pkgconf/system.h>             // System-wide configuration info
#include CYGBLD_HAL_VARIANT_H           // Variant (SA11x0) specific configuration
#include CYGBLD_HAL_PLATFORM_H          // Platform specific configuration
#include <cyg/hal/hal_sa11x0.h>         // Variant specific hardware definitions
#include <cyg/hal/hal_mmu.h>            // MMU definitions
#include <cyg/hal/blue_2g.h>            // Platform specific hardware definitions

#if defined(CYG_HAL_STARTUP_ROM)
#define PLATFORM_SETUP1 _platform_setup1
#define CYGHWR_HAL_ARM_HAS_MMU
// inv_xde
#define CYGSEM_HAL_ROM_RESET_USES_JUMP

#if (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 59000)
#define SA11X0_PLL_CLOCK 0x0        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 73700)
#define SA11X0_PLL_CLOCK 0x1
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 88500)
#define SA11X0_PLL_CLOCK 0x2        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 103200) 
#define SA11X0_PLL_CLOCK 0x3        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 118000)
#define SA11X0_PLL_CLOCK 0x4        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 132700)
#define SA11X0_PLL_CLOCK 0x5        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 147500)
#define SA11X0_PLL_CLOCK 0x6        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 162200)
#define SA11X0_PLL_CLOCK 0x7        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 176900)
#define SA11X0_PLL_CLOCK 0x8        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 191700)
#define SA11X0_PLL_CLOCK 0x9        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 206400)
#define SA11X0_PLL_CLOCK 0xA        
#elif (CYGHWR_HAL_ARM_SA11X0_PROCESSOR_CLOCK == 221200)
#define SA11X0_PLL_CLOCK 0xB        
#else
#error Invalid processor clock speed
#endif                
        
        
// This macro represents the initial startup code for the platform        
        .macro  _platform_setup1

        // Set up GPIO
        ldr     r1,=SA11X0_GPIO_PIN_DIRECTION
	ldr     r2, =0
        orr     r2,r2,#SA1110_GPIO_LED1
        orr     r2,r2,#SA1110_GPIO_LED2
        orr     r2,r2,#SA1110_GPIO_LED3
        orr     r2,r2,#SA1110_GPIO_LED4
        orr     r2,r2,#SA1110_GPIO_ADSL_RESET
        str     r2,[r1]

        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_CLEAR
	ldr     r2, =0
        orr     r2,r2,#SA1110_GPIO_LED1
        orr     r2,r2,#SA1110_GPIO_LED2
        orr     r2,r2,#SA1110_GPIO_LED3
        orr     r2,r2,#SA1110_GPIO_LED4
        orr     r2,r2,#SA1110_GPIO_ADSL_RESET
        str     r2,[r1]

        // Set up PCM GPIO
        ldr     r1,=SA11X0_PPC_PIN_DIRECTION
	ldr     r2, =0
        orr     r2,r2,#SA1110_PCM_ETHERNET_RESET
        bic     r2,r2,#SA1110_PCM_USB_RESET
        orr     r2,r2,#SA1110_PCM_PCMCIA1_RESET
        orr     r2,r2,#SA1110_PCM_DECT_RESET
        str     r2,[r1]

        // Pause
        ldr     r1,=10000
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

	// Turn LED1 on
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_SET
	ldr     r2, =0
        orr     r2,r2,#SA1110_GPIO_LED1
        str     r2,[r1]

        // Disable clock switching
        mcr     p15,0,r0,\
                SA11X0_TEST_CLOCK_AND_IDLE_REGISTER,\
                SA11X0_DISABLE_CLOCK_SWITCHING_RM,\
                SA11X0_DISABLE_CLOCK_SWITCHING_OPCODE

        // Set up processor clock
        ldr     r1,=SA11X0_PWR_MGR_PLL_CONFIG
        ldr     r2,=SA11X0_PLL_CLOCK
        str     r2,[r1]

        // Turn clock switching back on
        mcr     p15,0,r0,\
                SA11X0_TEST_CLOCK_AND_IDLE_REGISTER,\
                SA11X0_ENABLE_CLOCK_SWITCHING_RM,\
                SA11X0_ENABLE_CLOCK_SWITCHING_OPCODE
        nop
        nop


        // Initialize DRAM controller
        ldr     r1,=dram_table
        ldr     r2,=__exception_handlers
        sub     r1,r1,r2
        ldr     r2,[r1],#4                      // First control register
10:     ldr     r3,[r1],#4
        str     r3,[r2]
        ldr     r2,[r1],#4                      // Next control register
        cmp     r2,#0
        bne     10b

        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

	// Turn LED2 on
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_SET
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED2
        str     r2,[r1]

        // Enable UART
        ldr     r1,=SA1110_GPCLK_CONTROL_0
        ldr     r2,=SA1110_GPCLK_SUS_UART
        str     r2,[r1]

        // Release DRAM hold (set by RESET)
        ldr     r1,=SA11X0_PWR_MGR_SLEEP_STATUS
        ldr     r2,=SA11X0_DRAM_CONTROL_HOLD
        str     r2,[r1]

        // Perform 8 reads from unmapped/unenabled DRAM
        ldr     r1,=SA11X0_RAM_BANK0_BASE
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]
        ldr     r2,[r1]

        // Enable DRAM controller
        ldr     r1,=SA11X0_DRAM_CONFIGURATION
	ldr     r2,=0x0000B135
        str     r2,[r1]

        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

	// Turn LED3 on
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_SET
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED3
        str     r2,[r1]

        b       19f
        
/* Suggested Register Configurations:

SDRAM : Samsung K4S641632D-TC80
sdram bank 0, 32bit, 12row add bits, tRP=20ns, tRCD=20ns, tCK=8ns, tDPL=2clks, tCC=3clks, tREF=64ms, delay=500ns

nCS0 : tRC=120ns, tWP=50ns, tCHZ=25ns (AMD AM29DL640D)
nCS1 : tRC=50ns, tWP=25ns, tCHZ=20ns (Samsung K9F6408)
nCS2 : tRC=200ns, tWP=150ns, tCHZ=20ns (Altera EP1K100)  vrifier
nCS3 : tRC=500ns(497), tWP=20ns, tCHZ=20ns (Philips PDIUSBD12)
nCS4 : tRC=250ns, tWP=200ns, tCHZ=20ns (STM 20156)  vrifier

MDCNFG = 00000000000000001011000100110101 = 0x0000B135
SMCNFG = 00000000000000000000000000000000 = 0x00000000
MDREFR = 00000000001100000000001000000001 = 0x00300201
MDCAS00 = 01010101010101010101010101010111 = 0x55555557
MDCAS01 = 01010101010101010101010101010101 = 0x55555555
MDCAS02 = 01010101010101010101010101010101 = 0x55555555
MSC0 = 00100001000101000010001100110000 = 0x21142330
MSC1 = 00100001111111000010100101100100 = 0x21FC2964
MSC2 = 00000000000000000010110101111100 = 0x00002D7C

*/
 
// DRAM controller initialization        
dram_table:
        .word   SA11X0_DRAM0_CAS_0,           0x55555557
        .word   SA11X0_DRAM0_CAS_1,           0x55555555
        .word   SA11X0_DRAM0_CAS_2,           0x55555555
        .word   SA11X0_STATIC_CONTROL_0,      0x21142330
        .word   SA11X0_STATIC_CONTROL_1,      0x21FC2964
        .word   SA11X0_STATIC_CONTROL_2,      0x00002D7C
        .word   SA11X0_REFRESH_CONFIGURATION, 0x00300201
        .word   SA11X0_SMROM_CONFIGURATION,   0x00000000
        .word   SA11X0_DRAM_CONFIGURATION,    0x0000B134        // Disabled
        .word   0, 0
19:

        // Wakeup from deep sleep mode
        ldr     r1,=SA11X0_RESET_STATUS
        ldr     r2,[r1]
        cmp     r2,#SA11X0_SLEEP_MODE_RESET
        bne     45f
        ldr     r1,=SA11X0_PWR_MGR_SCRATCHPAD
        ldr     r1,[r1]
        mov     pc,r1
        nop
45:     nop

        // Release peripheral hold (set by RESET)
        ldr     r1,=SA11X0_PWR_MGR_SLEEP_STATUS
        ldr     r2,=SA11X0_PERIPHERAL_CONTROL_HOLD
        str     r2,[r1]

        // Set up a stack [for calling C code]
	ldr     r1,=__startup_stack
        ldr     r2,=SA11X0_RAM_BANK0_BASE
        orr     sp,r1,r2

        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

	// Turn LED4 on
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_SET
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED4
        str     r2,[r1]

        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

     	// Turn LED1 2 3 4 off
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_CLEAR
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED1
        orr     r2,r2,#SA1110_GPIO_LED2
        orr     r2,r2,#SA1110_GPIO_LED3
        orr     r2,r2,#SA1110_GPIO_LED4
        str     r2,[r1]


        // Create MMU tables
	bl      hal_mmu_init
       
        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

     	// Turn LED1 2 3 4 off
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_SET
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED1
        orr     r2,r2,#SA1110_GPIO_LED2
        orr     r2,r2,#SA1110_GPIO_LED3
        orr     r2,r2,#SA1110_GPIO_LED4
        str     r2,[r1]

        // Enable MMU
        ldr     r2,=10f
       	ldr	r1,=MMU_Control_Init|MMU_Control_M
	mcr	MMU_CP,0,r1,MMU_Control,c0
	mov	pc,r2    /* Change address spaces */
	nop
 	nop
	nop
10:
       
        // Pause
        ldr     r1,=LED_TIME
10:     sub     r1,r1,#1
        cmp     r1,#0
        bne     10b        

	// Turn LED1 2 3 4 off
        ldr     r1,=SA11X0_GPIO_PIN_OUTPUT_CLEAR
	ldr     r2,=0
        orr     r2,r2,#SA1110_GPIO_LED1
        orr     r2,r2,#SA1110_GPIO_LED2
        orr     r2,r2,#SA1110_GPIO_LED3
        orr     r2,r2,#SA1110_GPIO_LED4
        str     r2,[r1]

        .endm
        
#else // defined(CYG_HAL_STARTUP_ROM)
#define PLATFORM_SETUP1
#endif

/*---------------------------------------------------------------------------*/
/* end of hal_platform_setup.h                                               */
#endif /* CYGONCE_HAL_PLATFORM_SETUP_H */
